import os
import subprocess


def call_and_error(seq, and_print=False):
    """Use Popen to execute `seq` and return stdout."""
    if isinstance(seq, basestring):
        seq = seq.split()
    return subprocess.Popen(
        seq,
        stdout=subprocess.PIPE,
        stderr=and_print and subprocess.STDOUT or subprocess.PIPE
    ).communicate()


def call(seq, and_print=False):
    """Use Popen to execute `seq` and return stdout."""
    if isinstance(seq, basestring):
        seq = seq.split()
    return call_and_error(seq, and_print=and_print)[0]


def get_current_branchname():
    branches = get_branches()
    return [
        x.replace('* ', '').strip()
        for x in branches.splitlines()
        if x.startswith('* ')
    ][0]


def get_branches():
    return call(['git', 'branch'])


def get_repo_name():
    # `git rev-parse --show-toplevel`
    d = call('git rev-parse --show-toplevel'.split())
    return os.path.split(d)[-1].strip()



import os
import json

from . import config
from . import utils

from .merge import load


def run(force=False):
    branchname = utils.get_current_branchname()
    data = load(branchname)

    if force:
        if branchname == 'master' or config.FORK_REMOTE_NAME == 'origin':
            print "ERROR! I don't dare force push to that!"
            return 1

    cmd = ['git', 'push']
    if force:
        cmd.append('--force')
    cmd += [config.FORK_REMOTE_NAME, branchname]

    out, err = utils.call_and_error(cmd)
    print out
    print err

    return 0



import os
import json

from . import config
from . import utils


def get_repo_name():
    # `git rev-parse --show-toplevel`
    d = utils.call('git rev-parse --show-toplevel'.split())
    return os.path.split(d)[-1].strip()


def load(branchname):
    data = json.load(open(os.path.expanduser(config.SAVE_FILE)))
    repo_name = get_repo_name()
    try:
        return data['%s:%s' % (repo_name, branchname)]
    except KeyError:
        # possibly one of the old ones
        return data[branchname]


def run():
    branchname = utils.get_current_branchname()
    data = load(branchname)

    if data.get('gitflow'):
        print utils.call("git checkout develop", and_print=True)
        print utils.call("git pull origin develop", and_print=True)
        print utils.call(['git', 'checkout', branchname], and_print=True)
        #print utils.call("git flow feature rebase -i".split())
        print "Now run:\n"
        print "    ", "git flow feature rebase -i"
    else:

        print utils.call("git checkout master", and_print=True)
        print utils.call("git pull origin master", and_print=True)
        print utils.call(['git', 'checkout', branchname], and_print=True)
        #print utils.call("git rebase -i master".split())
        print "Now run:\n"
        print "    ", "git rebase -i master"
    print



import os
import json

from . import config
from . import utils
from . import merge


def run():
    branchname = utils.get_current_branchname()
    if branchname == 'master':
        print "ERROR. You're already on the master branch."
        exit()
    data = merge.load(branchname)
    _status = utils.call(['git', 'status', '--porcelain'])
    changed = [x for x in _status.splitlines() if not x.startswith('?? ')]
    if changed:
        print "ERROR. Some still changed files"
        print "\t", "\n\t".join(changed)
        exit()
    if data.get('gitflow'):
        print utils.call('git checkout develop')
        print utils.call('git pull origin develop')
    else:
        print utils.call('git checkout master')
        print utils.call('git pull origin master')
    _merged = [
        x.strip() for x in
        utils.call('git branch --merged').splitlines()
        if x.strip() and not x.strip().startswith('*')
    ]
    if branchname in _merged:
        # branch_delete = raw_input(
        #     "Delete merged branch '%s'? [Y/n]" % branchname
        # ).lower().strip()
        if 1: #branch_delete not in ('n', 'no'):
            # print ' '.join(['git', 'branch', '-d', branchname])
            print utils.call(['git', 'branch', '-d', branchname])
            # remote_delete = raw_input(
            #     "Delete remote fork branch too? [Y/n]"
            # ).lower().strip()
            if 1: #remote_delete not in ('n', 'no'):
                print utils.call(
                    ['git', 'push', config.FORK_REMOTE_NAME, ':%s' % branchname]
                )
                # print ' '.join(['git', 'push', config.FORK_REMOTE_NAME, ':%s' % branchname])



import operator
import subprocess
import datetime
from email.utils import parsedate_tz

def call(seq):
    if isinstance(seq, basestring):
        seq = seq.split()
    return subprocess.Popen(seq,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT).communicate()[0]


def _parse_branch_info(output):
    info = {}
    commit_msg = None
    for thing in output.splitlines():
        if commit_msg is not None:
            commit_msg += thing + '\n'
            continue
        if thing.startswith('commit'):
            continue
        if thing == '':
            commit_msg = ''
            continue
        label, stuff = thing.split(':', 1)
        info[label.strip()] = stuff.strip()

    info['msg'] = commit_msg
    return info


def branch_info(name):
    output = call('git log -n 1 %s' % name)
    info = _parse_branch_info(output)

    info['name'] = name
    info['date'] = info['Date']
    info['dt'] = parsedate_tz(info['date'])
    info['dt'] = datetime.datetime(*info['dt'][:7])
    age = datetime.datetime.now() - info['dt']

    info['age'] = age
    return info


def find(search):
    refs = call('git for-each-ref --shell refs/')
    branches = []
    for ref in refs.splitlines():
        try:
            branch_name = ref.split('refs/heads/')[1][:-1]
        except IndexError:
            continue
        if search and search.lower() not in branch_name.lower():
            continue
        branches.append(branch_info(branch_name))
    return branches


def get_merged_branches():
    merged = []
    for line in call('git branch --merged').splitlines():
        line = line.strip()
        if not line.startswith('*'):
            # it can't be the one your on
            merged.append(line)
    return merged


def checkout(branch_name):
    call('git checkout %s' % branch_name)


def print_list(branches, merged):
    for each in sorted(branches, key=operator.itemgetter('dt')):
        print '-' * 79
        is_merged = " (MERGED ALREADY!)" if each['name'] in merged else ''
        print each['name'], is_merged
        print "\t", each['date']
        print "\t", each['age'], "ago"
        print "\t", each['msg'].strip()
        print


def run(searchstring):
    branches_ = find(searchstring)
    if branches_:
        merged = get_merged_branches()
        print "Found existing branches..."
        print_list(branches_, merged)
        if len(branches_) == 1:
            branch_name = branches_[0]['name']
            if len(branch_name) > 50:
                branch_name = branch_name[:47] + '...'
            check_it_out = raw_input(
                "Check out '%s'? [Y/n] " % branch_name
            )
            if check_it_out.lower().strip() != 'n':
                checkout(branches_[0]['name'])
    elif searchstring:
        print "Found no branches matching: %s" % searchstring
    else:
        print "Found no branches"



import os
import json
import operator

from . import config
from . import utils
from . import merge
from . import branches


def run(search):
    if not search:
        print "ERROR. You must supply a search term"
        exit()

    branches_ = branches.find(search)
    branches_ = sorted(branches_, key=operator.itemgetter('dt'))
    if not branches_:
        print "ERROR. No branch matched by search"
        exit()
    elif len(branches_) > 1:
        if search in [x['name'] for x in branches_]:
            # one of them was an exact match
            branches_ = [x for x in branches_ if x['name'] == search]
        else:
            # multiple and no exact match
            branches.print_list(
                branches_,
                branches.get_merged_branches()
            )
            print "ERROR. More than one match"
            exit()

    branch = branches_[0]
    data = merge.load(branch['name'])

    current = utils.get_current_branchname()
    return_to = None
    if (
        (data.get('gitflow') and current != 'develop') or
        (not data.get('gitflow') and current != 'master')
    ):
        return_to = current
        if data.get('gitflow'):
            print utils.call('git checkout develop')
        else:
            print utils.call('git checkout master')

    if data.get('gitflow'):
        print utils.call('git pull origin develop')
    else:
        print utils.call('git pull origin master')

    _merged = [
        x.strip() for x in
        utils.call('git branch --merged').splitlines()
        if x.strip() and not x.strip().startswith('*')
    ]
    branchname = branch['name']
    if branchname in _merged:
        # branch_delete = raw_input(
        #     "Delete merged branch '%s'? [Y/n]" % branchname
        # ).lower().strip()
        if 1: #branch_delete not in ('n', 'no'):
            print utils.call(['git', 'branch', '-d', branchname])
            # remote_delete = raw_input(
            #     "Delete remote fork branch too? [Y/n]"
            # ).lower().strip()
            if 1: #remote_delete not in ('n', 'no'):
                print utils.call(
                    ['git', 'push', config.FORK_REMOTE_NAME, ':%s' % branchname]
                )
    else:
        print "%s can't be deleted because it's not merged" % _merged

    if return_to:
        print utils.call(['git', 'checkout', return_to])



import sys
from lib import (
    start,
    branches,
    commit,
    merge,
    rebase,
    makediff,
    getback,
    cleanup,
    push,
    tags,
)


def run():
    try:
        r = _run()
    except KeyboardInterrupt:
        r = 0
        print
    return int(r or 0)


def _run():
    import argparse
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    #
    # Branches
    #

    def run_branches(args):
        branches.run(args.searchstring)

    p = subparsers.add_parser(
        'branches',
        help="List and search active branches"
    )
    p.set_defaults(func=run_branches)
    p.add_argument('searchstring', nargs='?', help='Branch search string')

    #
    # Start
    #
    def run_start(args):
        start.run(args.bugid)

    p = subparsers.add_parser('start', help="Start a new branch")
    p.set_defaults(func=run_start)
    p.add_argument('bugid', nargs='?', help='Optional Bugzilla ID')

    #
    # Commit
    #
    def run_commit(args):
        if args.no_verify:
            commit.commit_all('--no-verify')
        else:
            commit.commit_all()

    p = subparsers.add_parser('commit', help="Commit the current branch")
    p.set_defaults(func=run_commit)
    p.add_argument("--no-verify",
                   help="Makes sense if you use check.py",
                   action="store_true")

    #
    # Rebase
    #
    def run_rebase(args):
        rebase.run()

    p = subparsers.add_parser(
        'rebase', help="Pull latest master and rebase against that"
    )
    p.set_defaults(func=run_rebase)

    #
    # Merge
    #
    def run_merge(args):
        merge.run()

    p = subparsers.add_parser(
        'merge', help="Merge the current branch onto master"
    )
    p.set_defaults(func=run_merge)

    #
    # Makediff
    #
    def run_makediff(args):
        makediff.run()

    p = subparsers.add_parser('makediff', help="Make a diff file")
    p.set_defaults(func=run_makediff)

    #
    # Getback
    #
    def run_getback(args):
        getback.run()

    p = subparsers.add_parser(
        'getback',
        help="Go back to master, update and clean up the branch you came from")
    p.set_defaults(func=run_getback)

    #
    # Cleanup
    #
    def run_cleanup(args):
        cleanup.run(args.searchstring)

    p = subparsers.add_parser(
        'cleanup',
        help="Delete a merged branch locally and remotely"
    )
    p.set_defaults(func=run_cleanup)
    p.add_argument('searchstring', nargs='?', help='Branch search string')

    #
    # Push
    #
    def run_push(args):
        push.run(args.force)

    p = subparsers.add_parser(
        'push',
        help="Push accordingly"
    )
    p.set_defaults(func=run_push)
    p.add_argument(
        '-f', '--force', action='store_true', help='Optionally force push'
    )

    #
    # Tag
    #
    def run_tag(args):
        tags.run()

    p = subparsers.add_parser(
        'tag',
        help="Create and push a new tag"
    )
    p.set_defaults(func=run_tag)

    #
    #
    #

    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    sys.exit(run())



import re

from . import utils


def run():
    branchname = utils.get_current_branchname()
    if branchname != 'master':
        print "Switching to the master branch"
        out, err = utils.call_and_error([
            'git', 'checkout', 'master'
        ])
        print out
        print err
    out, err = utils.call_and_error(
        "git for-each-ref --sort=taggerdate "
        "--format %(refname) refs/tags".split()
    )
    latest_tag = out.strip().splitlines()[-1].replace('refs/tags/', '')
    print "Last tag was:", latest_tag

    new_tag = re.sub(
        '(\d+)(?!\d)',
        lambda x: str(int(x.group(0)) + 1),
        latest_tag
    )
    new_tag_input = raw_input("New tag ['%s']: " % new_tag).strip()
    new_tag = new_tag_input or new_tag
    # print "You want to create a new tag:", new_tag

    default_msg = 'Release %s' % new_tag
    msg = raw_input("Commit message ['%s']: " % default_msg)
    msg = msg or default_msg

    out, err = utils.call_and_error([
        'git', 'tag', '-a', new_tag,
        '-m', msg
    ])
    print out
    print err

    out, err = utils.call_and_error([
        'git', 'push', '--tags',
    ])
    print out
    print err



# -*- coding: utf-8 -*-

import os
import re
import stat
import datetime
import json

from . import config
from . import utils


def save(description, branchname, bugnumber):
    data = json.load(open(os.path.expanduser(config.SAVE_FILE)))
    if branchname in data:
        # old style
        key = branchname
    else:
        key = '%s:%s' % (utils.get_repo_name(), branchname)
    data[key] = {'description': description, 'bugnumber': bugnumber}
    json.dump(open(os.path.expanduser(config.SAVE_FILE), 'w'), indent=2)


def load(branchname):
    data = json.load(open(os.path.expanduser(config.SAVE_FILE)))
    try:
        return data['%s:%s' % (utils.get_repo_name(), branchname)]
    except KeyError:
        return data[branchname]


def _find_git_root(dir_):
    dir_ = os.path.normpath(dir_)
    if dir_ == '/':
        raise OSError("Can't find .git/ parent")
    if os.path.isdir(os.path.join(dir_, '.git')):
        return dir_
    return _find_git_root(os.path.join(dir_, '..'))


def commit_all(*args):
    branchname = utils.get_current_branchname()
    data = load(branchname)

    _status = utils.call(['git', 'status', '--porcelain'])
    unstaged = [
        x.replace('?? ', '').strip()
        for x in _status.splitlines()
        if x.startswith('?? ')
    ]
    if unstaged:
        print "NOTE! There are untracked files:"
        _root = _find_git_root(os.curdir)
        _now = datetime.datetime.now()
        _youngest_new_file = None
        for each in unstaged:
            filepath = os.path.join(_root, each)
            try:
                ts = os.stat(filepath)[stat.ST_MTIME]
                mod_time = datetime.datetime.fromtimestamp(ts)
                print ' ' * 4,
                print filepath.replace(_root, '').ljust(60),
                _age = _now - mod_time
                if _youngest_new_file is None:
                    _youngest_new_file = _age
                elif _age < _youngest_new_file:
                    _youngest_new_file = _age
                age = str(_age)
                if ' days,' in age:
                    age = re.findall('\d+ days', age)[0]
                else:
                    age = re.sub(':(\d\d)\.\d+', r':\1', age)
            except OSError:
                age = 'n/a'
            print age

        if _youngest_new_file and _youngest_new_file.total_seconds() < 60 * 60 * 12:
            # if a file has been added in the last 12 hours, we have to ask
            # this question
            ignore = raw_input("Ignore untracked files? [Y/n]").lower().strip()
            if ignore.lower().strip() == 'n':
                print (
                    "\n\tLeaving it to you to figure out what to do with "
                    "the untracked files"
                )
                return 1
            print

    if data['bugnumber']:
        msg = 'bug %s - %s' % (data['bugnumber'], data['description'])
    else:
        msg = data['description']

    print "MSG:"
    print "\t", msg
    print
    confirm = raw_input("OK? [Y/n] ").lower().strip()
    if confirm in ('n', 'no'):
        try_again = raw_input("Type your own (or empty to exit): ")
        if not try_again:
            return 1
        msg = try_again

    if data['bugnumber']:
        fixes_prefix = raw_input(
            "Add the 'fixes ' prefix? [N/y] "
        ).lower().strip()
        if fixes_prefix in ('y', 'yes'):
            msg = 'fixes %s' % msg

    cmd = ['git', 'commit', '-a', '-m', msg]
    #cmd = ['git', 'commit', '-m', msg]
    if '--no-verify' in args:
        cmd.append('--no-verify')
    out, err = utils.call_and_error(cmd)
    if err and err.strip():
        print err
        return 1
    print "NOW, feel free to run:\n"
    print "git checkout master"
    print "git merge %s" % branchname
    print "git branch -d %s" % branchname
    print
    print "OR\n"
    print "git push %s %s" % (config.FORK_REMOTE_NAME, branchname)
    print
    push_for_you = raw_input("Run that push? [Y/n] ").lower().strip()
    if push_for_you not in ('n', 'no'):
        cmd = ['git', 'push', config.FORK_REMOTE_NAME, branchname]
        out, err = utils.call_and_error(cmd)
        if err and err.strip():
            print err

        out, err = utils.call_and_error('git remote show origin')
        if err:
            print "No remote called 'origin'"
            return 0

        print "Now, to make a new Pull Request, go to:"
        print
        for line in out.splitlines():
            if line.strip().startswith('Push  URL'):
                rest = re.split('github\.com[:/]', line)[1]
                org, repo = rest.split('.git')[0].split('/', 1)
                break
        url = "https://github.com/%s/%s/compare/%s:%s...%s:%s?expand=1" % (
            org,
            repo,
            org,
            'master',
            config.FORK_REMOTE_NAME,
            branchname,
        )
        print url
        print "(⌘-click to open URLs)"
        print

    return 0



import re
import datetime
import os
import json
import subprocess
import urllib
import urllib2
import getpass
import cookielib

from . import config
from . import utils

bugzilla_url_regex = reg = re.compile(
    re.escape('https://bugzilla.mozilla.org/show_bug.cgi?id=') + '(\d+)$'
)


def get_bugzilla_summary(bugnumber):
    credentials = config.BUGZILLA_CREDENTIALS
    if not credentials:
        credentials = None
        bugzilla_login = raw_input('Bugzilla username: ')
        if bugzilla_login:
            bugzilla_password = getpass.getpass('Bugzilla password: ').strip()
            if bugzilla_password:
                login_payload = {
                    'Bugzilla_login': bugzilla_login,
                    'Bugzilla_password': bugzilla_password,
                    'Bugzilla_remember': 'on',
                    'GoAheadAndLogIn': 'Log in'
                }
                data = urllib.urlencode(login_payload)
                cookies = cookielib.LWPCookieJar()
                handlers = [
                    urllib2.HTTPHandler(),
                    urllib2.HTTPSHandler(),
                    urllib2.HTTPCookieProcessor(cookies)
                ]
                opener = urllib2.build_opener(*handlers)
                req = urllib2.Request(config.BUGZILLA_LOGIN_URL, data)
                response = opener.open(req)
                credentials = {'username': bugzilla_login}
                for cookie in cookies:
                    credentials[cookie.name] = cookie.value
                config.save_bugzilla_credentials(credentials)
                print "Your bugzilla cookie is now stored in",
                print config.CONFIG_FILE

    headers = {'Accept': 'application/json'}
    url = 'https://api-dev.bugzilla.mozilla.org/latest/bug/%s' % bugnumber
    request_arguments = {}
    if credentials:
        request_arguments['cookie'] = credentials['Bugzilla_login_request_cookie']

    data = urllib.urlencode(request_arguments)
    if data:
        url += '?%s' % data
    req = urllib2.Request(url, headers=headers)
    try:
        response = urllib2.urlopen(req)
    except urllib2.URLError:
        import logging
        logging.error('Failed to connect to Bugzilla', exc_info=True)
        return ''
    assert response.code == 200, '%s - %s' % (response.code, response.read())
    the_page = response.read()
    struct = json.loads(the_page)
    return struct['summary']


def get_repo_name():
    # `git rev-parse --show-toplevel`
    d = call('git rev-parse --show-toplevel'.split())
    return os.path.split(d)[-1].strip()


def save(description, branchname, bugnumber, gitflow=False):
    try:
        data = json.load(open(os.path.expanduser(config.SAVE_FILE)))
    except IOError:
        data = {}
    repo_name = get_repo_name()
    data['%s:%s' % (repo_name, branchname)] = {
        'description': description,
        'bugnumber': bugnumber,
        'gitflow': gitflow,
        'date': datetime.datetime.now().isoformat()
    }
    json.dump(data, open(os.path.expanduser(config.SAVE_FILE), 'w'), indent=2)


def call(seq):
    """Use Popen to execute `seq` and return stdout."""
    return subprocess.Popen(seq,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT).communicate()[0]


def run(bugnumber=None):
    branches = utils.get_branches()
    if 'Not a git repository' in branches:
        print "Are you sure you're in a git repository?"
        return 1

    current_branchname = utils.get_current_branchname()
    print "You're currently on branch", current_branchname
    if bugnumber:
        # perhaps the user was lazy any typed in something like a full
        # url to a bug. If so, get just the bug number
        if bugzilla_url_regex.findall(bugnumber.split('#')[0]):
            bugnumber = bugzilla_url_regex.findall(bugnumber.split('#')[0])[0]
        if not bugnumber.isdigit():
            raise ValueError("%r doesn't look like a bug number")
        summary = get_bugzilla_summary(bugnumber)
        summary = re.sub('^\[[\w-]+\]\s*', '', summary)
    else:
        summary = None
    if summary:
        description = raw_input('Summary ["%s"]: ' % summary).strip()
        if not description:
            description = summary
    else:
        description = raw_input("Summary: ").strip()

    branchname = ''
    if bugnumber:
        branchname = 'bug-%s-' % bugnumber

    def clean_branchname(string):
        string = (
            string
            .replace('   ', ' ')
            .replace('  ', ' ')
            .replace(' ', '-')
            .replace('=', '-')
            .replace('->', '-')
            .replace('---', '-')
        )
        for each in ':\'"/(),[]{}.?`$<>#*;':
            string = string.replace(each, '')
        return string.lower().strip()

    branchname += clean_branchname(description)
    if current_branchname == 'develop':
        print "(assuming this is a git flow repo)"
        print call(['git', 'flow', 'feature', 'start', branchname])
        branchname = 'feature/%s' % branchname
    else:
        print call(['git', 'checkout', '-b', branchname])
    save(description, branchname, bugnumber,
         gitflow=current_branchname == 'develop')



import os
import json
import subprocess

from . import config


def get_repo_name():
    # `git rev-parse --show-toplevel`
    d = call('git rev-parse --show-toplevel'.split())
    return os.path.split(d)[-1].strip()


def load(branchname):
    data = json.load(open(os.path.expanduser(config.SAVE_FILE)))
    repo_name = get_repo_name()
    try:
        return data['%s:%s' % (repo_name, branchname)]
    except KeyError:
        # possibly one of the old ones
        return data[branchname]


def call(seq):
    """Use Popen to execute `seq` and return stdout."""
    return subprocess.Popen(seq,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT).communicate()[0]


def run():
    branches = call(['git', 'branch'])
    branchname = [
        x.replace('* ', '').strip()
        for x in branches.splitlines()
        if x.startswith('* ')
    ][0]
    data = load(branchname)

    def make_filename(bugnumber, increment=None):
        home = os.path.expanduser('~/tmp/')
        if increment:
            f = os.path.join(home, 'bug%s-%s.diff' % (bugnumber, increment))
        else:
            f = os.path.join(home, 'bug%s.diff' % bugnumber)
            increment = 1
        if os.path.isfile(f):
            return make_filename(bugnumber, increment + 1)
        return f

    filename = make_filename(data['bugnumber'])

    if branchname.startswith('feature/'):
        with open(filename, 'w') as f:
            f.write(call(['git', 'diff', 'develop...']))

    else:
        with open(filename, 'w') as f:
            f.write(call(['git', 'diff', 'master...']))

    print filename



import subprocess


def call(seq):
    if isinstance(seq, basestring):
        seq = seq.split()
    return subprocess.Popen(seq,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT).communicate()[0]


def _files():
    output = call('git status --porcelain')
    staged = []
    not_staged = []
    untracked = []
    for line in output.splitlines():
        info, path = line.split(' ', 1)
        if info.startswith('??'):
            untracked.append(path)
        elif info.startswith(' '):
            staged.append(path)
        else:
            not_staged.append(path)
    return staged, not_staged, untracked


def files_staged_or_not_staged():
    #output = call('git status --porcelain')
    staged, not_staged, untracked = _files()
    return staged + not_staged


def print_status():
    print call('git status')


def current_branch():
    output = call('git branch')
    for line in output.splitlines():
        if line.startswith('* '):
            return line[2:]



import codecs
import os
import re

# Prevent spurious errors during `python setup.py test`, a la
# http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html:
try:
    import multiprocessing
except ImportError:
    pass

from setuptools import setup, find_packages


def read(*parts):
    return codecs.open(os.path.join(os.path.dirname(__file__), *parts)).read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def find_install_requires():
    return [x.strip() for x in
            read('requirements.txt').splitlines()
            if x.strip() and not x.startswith('#')]


setup(
    name='bgg',
    scripts=['G'],
    version=find_version('bgg/__init__.py'),
    description="A love story with Bugzilla, git and Github",
    long_description=read('README.md') + '\n\n',
    author='Peter Bengtsson',
    author_email='mail@peterbe.com',
    url='https://github.com/peterbe/bgg',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    install_requires=find_install_requires(),
)



import os
import json

from . import config
from . import utils


def save(branchname):
    data = json.load(open(os.path.expanduser(config.SAVE_FILE)))
    repo_name = utils.get_repo_name()
    try:
        del data['%s:%s' % (repo_name, branchname)]
    except KeyError:
        # legacy
        del data[branchname]
    json.dump(data, open(os.path.expanduser(config.SAVE_FILE), 'w'), indent=2)


def load(branchname):
    data = json.load(open(os.path.expanduser(config.SAVE_FILE)))
    repo_name = utils.get_repo_name()
    try:
        return data['%s:%s' % (repo_name, branchname)]
    except KeyError:
        # possibly one of the old ones
        return data[branchname]


def run():
    branchname = utils.get_current_branchname()
    data = load(branchname)
    _status = utils.call(['git', 'status', '--porcelain'])
    changed = [x for x in _status.splitlines() if not x.startswith('?? ')]
    if changed:
        print "ERROR. Some still changed files"
        print "\t", "\n\t".join(changed)
        exit()

    last_force_push = raw_input(
        "One last forced push to %s? [N/y] " % config.FORK_REMOTE_NAME
    ).lower().strip()
    if last_force_push in ('y', 'yes'):
        utils.call(['git', 'push', '-f', config.FORK_REMOTE_NAME, branchname])

    if data.get('gitflow'):
        print utils.call(['git', 'flow', 'feature', 'finish', branchname.replace('feature/', '')])
    else:
        print utils.call("git checkout master".split())
        print utils.call(['git', 'merge', branchname])
        print utils.call(['git', 'branch', '-d', branchname])

    print "NOW, you might want to run:\n"
    if data.get('gitflow'):
        print "git push origin develop"
    else:
        print "git push origin master"
    print

    save(branchname)

    push_for_you = raw_input("Run that push? [Y/n] ").lower().strip()
    if push_for_you not in ('n', 'no'):
        cmd = ['git', 'push', 'origin']
        if data.get('gitflow'):
            cmd.append('develop')
        else:
            cmd.append('master')
        out, err = utils.call_and_error(cmd)
        if err and err.strip():
            print err



import json
import os

CONFIG_FILE = os.path.expanduser('~/.G.json')


def write(data):
    with open(CONFIG_FILE, 'w') as out:
        json.dump(data, out, indent=2)


def read():
    with open(CONFIG_FILE) as inp:
        return json.load(inp)


if not os.path.isfile(CONFIG_FILE):
    # make a default one
    write({
        'SAVE_FILE': '~/.Gstart.json',
        'BUGZILLA_LOGIN_URL': 'https://bugzilla.mozilla.org/index.cgi',
        'FORK_REMOTE_NAME': 'myforkname'
    })

config = read()
SAVE_FILE = config['SAVE_FILE']
BUGZILLA_CREDENTIALS = config.get('BUGZILLA_CREDENTIALS', {})
BUGZILLA_LOGIN_URL = config['BUGZILLA_LOGIN_URL']
FORK_REMOTE_NAME = config['FORK_REMOTE_NAME']


def save_bugzilla_credentials(credentials):
    config = read()
    config['BUGZILLA_CREDENTIALS'] = credentials
    write(config)
